@@ -3,8 +3,6 @@ from .jurisdiction import Jurisdiction, LegislativeSession
 from .division import Division
 from .people_orgs import (
     Organization,
-    OrganizationLink,
-    OrganizationSource,
     Person,
     PersonIdentifier,
     PersonName,
