@@ -1,7 +1,6 @@
 import datetime
 from django.db import models
 from django.db.models import Q, QuerySet
-from django.contrib.postgres.fields import JSONField
 from .base import OCDBase, LinkBase, OCDIDField, RelatedBase, IdentifierBase
 from .division import Division
 from .jurisdiction import Jurisdiction
@@ -41,6 +40,9 @@ class Organization(OCDBase):
         choices=common.ORGANIZATION_CLASSIFICATION_CHOICES,
         help_text="The type of Organization being defined.",
     )
+    sources = models.JSONField(default=list, blank=True)
+    links = models.JSONField(default=list, blank=True)
+    other_names = models.JSONField(default=list, blank=True)
 
     def __str__(self):
         return self.name
@@ -74,38 +76,6 @@ class Organization(OCDBase):
         ]
 
 
-class OrganizationLink(LinkBase):
-    """
-    URL for a document about an Organization.
-    """
-
-    organization = models.ForeignKey(
-        Organization,
-        related_name="links",
-        help_text="A reference to the Organization connected to this link.",
-        on_delete=models.CASCADE,
-    )
-
-    class Meta:
-        db_table = "opencivicdata_organizationlink"
-
-
-class OrganizationSource(LinkBase):
-    """
-    Source used in assembling an Organization.
-    """
-
-    organization = models.ForeignKey(
-        Organization,
-        related_name="sources",
-        help_text="A link to the Organization connected to this source.",
-        on_delete=models.CASCADE,
-    )
-
-    class Meta:
-        db_table = "opencivicdata_organizationsource"
-
-
 class Post(OCDBase):
     """
     A position in an organization that exists independently of the person holding it.
@@ -246,7 +216,7 @@ class Person(OCDBase):
         null=True,
         default=None,
     )
-    current_role = JSONField(null=True, default=None)
+    current_role = models.JSONField(null=True, default=None)
 
     def __str__(self):
         return self.name
