@@ -3,17 +3,19 @@ import copy
 import glob
 import json
 import logging
-from django.db.models import Q
+import typing
+from django.db.models import Q, Model
 from django.db.models.signals import post_save
-from django.contrib.contenttypes.models import ContentType
 from .. import settings
-from ..data.models import LegislativeSession
+from ..data.models import LegislativeSession, Person
 from ..exceptions import DuplicateItemError, UnresolvedIdError, DataImportError
-from ..reports.models import Identifier
 from ..utils import get_pseudo_id, utcnow
+from ._types import _ID, _JsonDict, _RelatedModels, _TransformerMapping
+
+_PersonCacheKey = typing.Tuple[str, typing.Optional[str], typing.Optional[str]]
 
 
-def omnihash(obj):
+def omnihash(obj: typing.Any) -> int:
     """ recursively hash unhashable objects """
     if isinstance(obj, set):
         return hash(frozenset(omnihash(e) for e in obj))
@@ -25,7 +27,33 @@ def omnihash(obj):
         return hash(obj)
 
 
-def items_differ(jsonitems, dbitems, subfield_dict):
+def _match(
+    dbitem: Model,
+    jsonitem: _JsonDict,
+    keys: typing.Iterable[str],
+    subfield_dict: typing.Dict[str, typing.Any],
+) -> bool:
+    # check if all keys (excluding subfields) match
+    for k in keys:
+        if k not in subfield_dict and getattr(dbitem, k) != jsonitem.get(k, None):
+            return False
+
+    # all fields match so far, possibly equal, just check subfields now
+    for k in subfield_dict:
+        jsonsubitems = jsonitem[k]
+        dbsubitems = list(getattr(dbitem, k).all())
+        if items_differ(jsonsubitems, dbsubitems, subfield_dict[k][2]):
+            return False
+
+    # if we got here, item values match
+    return True
+
+
+def items_differ(
+    jsonitems: typing.List[_JsonDict],
+    dbitems: typing.List[Model],
+    subfield_dict: _JsonDict,
+) -> bool:
     """ check whether or not jsonitems and dbitems differ """
 
     # short circuit common cases
@@ -43,30 +71,20 @@ def items_differ(jsonitems, dbitems, subfield_dict):
     # go over dbitems looking for matches
     for dbitem in dbitems:
         order = getattr(dbitem, "order", None)
+
         match = None
+
+        # if we have an order, we can just check one item
+        if order is not None:
+            # use original so that pop calls don't affect ordering
+            if not _match(dbitem, original_jsonitems[order], keys, subfield_dict):
+                # short circuit if there isn't a match in the right spot
+                return True
+
+        # need to get position of match to remove
         for i, jsonitem in enumerate(jsonitems):
-            # check if all keys (excluding subfields) match
-            for k in keys:
-                if k not in subfield_dict and getattr(dbitem, k) != jsonitem.get(
-                    k, None
-                ):
-                    break
-            else:
-                # all fields match so far, possibly equal, just check subfields now
-                for k in subfield_dict:
-                    jsonsubitems = jsonitem[k]
-                    dbsubitems = list(getattr(dbitem, k).all())
-                    if items_differ(jsonsubitems, dbsubitems, subfield_dict[k][2]):
-                        break
-                else:
-                    # if the dbitem sets 'order', then the order matters
-                    if order is not None and int(order) != original_jsonitems.index(
-                        jsonitem
-                    ):
-                        break
-                    # these items are equal, so let's mark it for removal
-                    match = i
-                    break
+            if _match(dbitem, jsonitem, keys, subfield_dict):
+                match = i
 
         if match is not None:
             # item exists in both, remove from jsonitems
@@ -83,8 +101,8 @@ def items_differ(jsonitems, dbitems, subfield_dict):
     return False
 
 
-class BaseImporter(object):
-    """ BaseImporter
+class BaseImporter:
+    """BaseImporter
 
     Override:
         get_object(data)
@@ -94,19 +112,20 @@ class BaseImporter(object):
         update_computed_fields(obj)     [optional]
     """
 
-    _type = None
-    model_class = None
-    related_models = {}
-    preserve_order = set()
-    merge_related = {}
-    cached_transformers = {}
+    _type: str
+    model_class: Model = None
+    related_models: _RelatedModels = {}
+    preserve_order: typing.Set[str] = set()
+    merge_related: typing.Dict[str, typing.List[str]] = {}
+    cached_transformers: _TransformerMapping = {}
 
-    def __init__(self, jurisdiction_id):
+    def __init__(self, jurisdiction_id: str) -> None:
         self.jurisdiction_id = jurisdiction_id
-        self.json_to_db_id = {}
-        self.duplicates = {}
-        self.pseudo_id_cache = {}
-        self.session_cache = {}
+        self.json_to_db_id: typing.Dict[str, _ID] = {}
+        self.duplicates: typing.Dict[str, str] = {}
+        self.pseudo_id_cache: typing.Dict[str, typing.Optional[_ID]] = {}
+        self.person_cache: typing.Dict[_PersonCacheKey, typing.Optional[str]] = {}
+        self.session_cache: typing.Dict[str, LegislativeSession] = {}
         self.logger = logging.getLogger("openstates")
         self.info = self.logger.info
         self.debug = self.logger.debug
@@ -118,36 +137,44 @@ class BaseImporter(object):
         if settings.IMPORT_TRANSFORMERS.get(self._type):
             self.cached_transformers = settings.IMPORT_TRANSFORMERS[self._type]
 
-    def get_session_id(self, identifier):
+    def get_session(self, identifier: str) -> LegislativeSession:
         if identifier not in self.session_cache:
             self.session_cache[identifier] = LegislativeSession.objects.get(
                 identifier=identifier, jurisdiction_id=self.jurisdiction_id
-            ).id
+            )
         return self.session_cache[identifier]
 
+    def limit_spec(self, spec: _JsonDict) -> _JsonDict:
+        raise NotImplementedError()
+
+    def get_object(self, object: _JsonDict) -> Model:
+        raise NotImplementedError()
+
     # no-ops to be overriden
-    def prepare_for_db(self, data):
+    def prepare_for_db(self, data: _JsonDict) -> _JsonDict:
         return data
 
-    def postimport(self):
+    def postimport(self) -> None:
         pass
 
-    def update_computed_fields(self, obj):
+    def update_computed_fields(self, obj: Model) -> None:
         pass
 
-    def resolve_json_id(self, json_id, allow_no_match=False):
+    def resolve_json_id(
+        self, json_id: str, allow_no_match: bool = False
+    ) -> typing.Optional[_ID]:
         """
-            Given an id found in scraped JSON, return a DB id for the object.
+        Given an id found in scraped JSON, return a DB id for the object.
 
-            params:
-                json_id:        id from json
-                allow_no_match: just return None if id can't be resolved
+        params:
+            json_id:        id from json
+            allow_no_match: just return None if id can't be resolved
 
-            returns:
-                database id
+        returns:
+            database id
 
-            raises:
-                ValueError if id couldn't be resolved
+        raises:
+            ValueError if id couldn't be resolved
         """
         if not json_id:
             return None
@@ -194,10 +221,10 @@ class BaseImporter(object):
         except KeyError:
             raise UnresolvedIdError("cannot resolve id: {}".format(json_id))
 
-    def import_directory(self, datadir):
+    def import_directory(self, datadir: str) -> typing.Dict[str, typing.Dict]:
         """ import a JSON directory into the database """
 
-        def json_stream():
+        def json_stream() -> typing.Iterator[_JsonDict]:
             # load all json, mapped by json_id
             for fname in glob.glob(os.path.join(datadir, self._type + "_*.json")):
                 with open(fname) as f:
@@ -205,9 +232,10 @@ class BaseImporter(object):
 
         return self.import_data(json_stream())
 
-    def _prepare_imports(self, dicts):
-
-        """ filters the import stream to remove duplicates
+    def _prepare_imports(
+        self, dicts: typing.Iterable[_JsonDict]
+    ) -> typing.Iterator[typing.Tuple[str, _JsonDict]]:
+        """filters the import stream to remove duplicates
 
         also serves as a good place to override if anything special has to be done to the
         order of the import stream (see OrganizationImporter)
@@ -228,7 +256,9 @@ class BaseImporter(object):
             else:
                 self.duplicates[json_id] = seen_hashes[objhash]
 
-    def import_data(self, data_items):
+    def import_data(
+        self, data_items: typing.Iterable[_JsonDict]
+    ) -> typing.Dict[str, typing.Dict]:
         """ import a bunch of dicts together """
         # keep counts of all actions
         record = {
@@ -252,7 +282,7 @@ class BaseImporter(object):
 
         return {self._type: record}
 
-    def import_item(self, data):
+    def import_item(self, data: _JsonDict) -> typing.Tuple[_ID, str]:
         """ function used by import_data """
         what = "noop"
 
@@ -270,9 +300,6 @@ class BaseImporter(object):
         except self.model_class.DoesNotExist:
             obj = None
 
-        # remove pupa_id which does not belong in the OCD data models
-        pupa_id = data.pop("pupa_id", None)
-
         # pull related fields off
         related = {}
         for field in self.related_models:
@@ -316,16 +343,14 @@ class BaseImporter(object):
             # for handlers make use of related objects
             post_save.send(sender=self.model_class, instance=obj, created=True)
 
-        if pupa_id:
-            Identifier.objects.get_or_create(
-                identifier=pupa_id,
-                jurisdiction_id=self.jurisdiction_id,
-                defaults={"content_object": obj},
-            )
-
         return obj.id, what
 
-    def _update_related(self, obj, related, subfield_dict):
+    def _update_related(
+        self,
+        obj: Model,
+        related: typing.Dict[str, typing.List[Model]],
+        subfield_dict: _JsonDict,
+    ) -> bool:
         """
         update DB objects related to a base object
             obj:            a base object to create related
@@ -392,7 +417,12 @@ class BaseImporter(object):
 
         return updated
 
-    def _create_related(self, obj, related, subfield_dict):
+    def _create_related(
+        self,
+        obj: Model,
+        related: typing.Dict[str, typing.List[Model]],
+        subfield_dict: _JsonDict,
+    ) -> None:
         """
         create DB objects related to a base object
             obj:            a base object to create related
@@ -434,20 +464,9 @@ class BaseImporter(object):
             for subobj, subrel in zip(subobjects, all_subrelated):
                 self._create_related(subobj, subrel, subsubdict)
 
-    def lookup_obj_id(self, pupa_id, model):
-        content_type = ContentType.objects.get_for_model(model)
-        try:
-            obj_id = Identifier.objects.get(
-                identifier=pupa_id,
-                content_type=content_type,
-                jurisdiction_id=self.jurisdiction_id,
-            ).object_id
-        except Identifier.DoesNotExist:
-            obj_id = None
-
-        return obj_id
-
-    def apply_transformers(self, data, transformers=None):
+    def apply_transformers(
+        self, data: _JsonDict, transformers: typing.Optional[_TransformerMapping] = None
+    ) -> _JsonDict:
         if transformers is None:
             transformers = self.cached_transformers
 
@@ -464,5 +483,76 @@ class BaseImporter(object):
 
         return data
 
-    def get_seen_sessions(self):
-        return self.session_cache.values()
+    def get_seen_sessions(self) -> typing.List[int]:
+        return [s.id for s in self.session_cache.values()]
+
+    def resolve_person(
+        self,
+        psuedo_person_id: str,
+        start_date: typing.Optional[str] = None,
+        end_date: typing.Optional[str] = None,
+        org_classification: typing.Optional[str] = None,
+    ) -> str:
+        cache_key = (psuedo_person_id, start_date, end_date)
+        if cache_key in self.person_cache:
+            return self.person_cache[cache_key]
+
+        # turn spec into DB query
+        spec = get_pseudo_id(psuedo_person_id)
+        if list(spec.keys()) == ["name"]:
+            # if we're just resolving on name, include other names and family name
+            name = spec["name"]
+            spec = (
+                Q(name__iexact=name)
+                | Q(other_names__name__iexact=name)
+                | Q(family_name__iexact=name)
+            )
+        else:
+            spec = Q(**spec)
+
+        spec &= Q(
+            memberships__organization__jurisdiction_id=self.jurisdiction_id,
+        )
+
+        if org_classification:
+            spec &= Q(memberships__organization__classification=org_classification)
+        else:
+            spec &= Q(
+                memberships__organization__classification__in=(
+                    "upper",
+                    "lower",
+                    "legislature",
+                )
+            )
+
+        # we don't know what dates we have available to us... it can be any configuration
+        # of start/end dates or they can all be null.  Instead of requiring a strict overlap
+        # we use them to exclude people that are definitely not serving in the session:
+        # if we know when the session started, ensure they are either in office still or
+        #   that their end date was after the start of the session
+        if start_date:
+            spec &= Q(memberships__end_date="") | Q(
+                memberships__end_date__gt=start_date
+            )
+        # if we know when the session ended, ensure that they didn't start after it ended
+        if end_date:
+            spec &= Q(memberships__start_date="") | Q(
+                memberships__start_date__lt=end_date
+            )
+
+        ids = set(Person.objects.filter(spec).values_list("id", flat=True))
+        if len(ids) == 1:
+            self.person_cache[cache_key] = ids.pop()
+            errmsg = None
+        elif not ids:
+            errmsg = f"no people returned for {spec}"
+        else:
+            errmsg = f"multiple people returned for {spec}"
+
+        # either raise or log error
+        if errmsg:
+            self.error(errmsg)
+            self.person_cache[cache_key] = None
+
+        # return the newly-cached object
+        return self.person_cache[cache_key]
