@@ -1,10 +1,10 @@
 import os
-import django
-from django import conf
-import dj_database_url
+import django  # type: ignore
+from django import conf  # type: ignore
+import dj_database_url  # type: ignore
 
 
-def init_django():  # pragma: no cover
+def init_django() -> None:  # pragma: no cover
     DATABASE_URL = os.environ.get(
         "DATABASE_URL", "postgis://openstates:openstates@localhost/openstates"
     )
